#define	F_CPU		8000000
#define	BAUD		9600
#define	IRfreq	38000

#include <stdio.h>
#include <avr\io.h>
#include <avr\interrupt.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <util/setbaud.h>
#include <util/delay.h>
#include <inttypes.h>
#include <ctype.h>

#define nop()  __asm__ __volatile__ ("nop" ::)


static int uart_putchar(char c, FILE *stream);
static int uart_getchar(FILE *stream);
static FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);
static FILE mystdin = FDEV_SETUP_STREAM(NULL, uart_getchar, _FDEV_SETUP_READ);
static int
uart_putchar(char c, FILE *stream)
{
  if (c == '\n')
    uart_putchar('\r', stream);
  loop_until_bit_is_set(UCSRA, UDRE);
  UDR = c;
  return 0;
}

static int
uart_getchar(FILE *stream)
{
  loop_until_bit_is_set(UCSRA, RXC);
  return UDR ;
}

int uart_dataready() {
  return UCSRA >> RXC & 1 ;
}

static void
uart_9600(void)
{
  stdout = &mystdout ;
  stdin = &mystdin ;
  UBRRH = UBRRH_VALUE ;
  UBRRL = UBRRL_VALUE ;
  #if USE_2X
    UCSRA |= (1 << U2X) ;
  #else
    UCSRA &= ~(1 << U2X) ;
  #endif
  UCSRB |= (1<<TXEN)|(1<<RXEN) ;
}

#include <compat/deprecated.h>

#define	LedIR_direct			DDRB
#define	LedIR_port				PORTB
#define	LedIR_bit					0

#define	LedRed_direct			DDRB
#define	LedRed_port				PORTB
#define	LedRed_bit				3

#define	LedGreen_direct		DDRB
#define	LedGreen_port			PORTB
#define	LedGreen_bit			5

#define	LedBlue_direct		DDRB
#define	LedBlue_port			PORTB
#define	LedBlue_bit				4

#define	ReceiverIR_direct	DDRD
#define	ReceiverIR_port		PIND
#define	ReceiverIR_bit		3


int main(){
	sbi(	LedIR_direct,	LedIR_bit	);	
	sbi(	LedRed_direct,	LedRed_bit	);	
	sbi(	LedGreen_direct,	LedGreen_bit	);	
	sbi(	LedBlue_direct,	LedBlue_bit	);	

	sbi(	LedIR_port,	LedIR_bit	);
	sbi(	LedRed_port,	LedRed_bit	);
	sbi(	LedGreen_port,	LedGreen_bit	);
	sbi(	LedBlue_port,	LedBlue_bit	);

	uart_9600();
	_delay_ms( 100 );
	printf( "exp_Servo!\n" );

	cbi(	LedRed_port,	LedRed_bit	);
	cbi(	LedGreen_port,	LedGreen_bit	);
	cbi(	LedBlue_port,	LedBlue_bit	);

	//TIMSK |= 1 << OCIE2 ;
	//OCR2 = F_CPU	/	( 2 * IRfreq ) ;

	//TCCR2 = 1 | ( 0 << WGM20 ) | ( 1 << WGM21 ) ;
	
	sei();

  volatile uint32_t x = 300 ;
  volatile uint32_t y = 50000 ;

	DDRB = 0xFF ;

	printf("x=%ld, y=%ld\n", x, y);
	while(1) {
		_delay_us( y );
		PORTB = 0xFF ;
		_delay_us( x );
		PORTB = 0 ;

    if( uart_dataready() ) {
			uint8_t c = getchar();
			c =  toupper( c );
			switch ( c ) {
				case 'Q': { x++; printf("x=%ld, y=%ld\n", x, y); break; };
				case 'A': { x--; printf("x=%ld, y=%ld\n", x, y); break; };
				case 'W': { y++; printf("x=%ld, y=%ld\n", x, y); break; };
				case 'S': { y--; printf("x=%ld, y=%ld\n", x, y); break; };
				case 'Z': { printf("Hi!\n"); break; }; 
			}
		}
	}
}
